FusionHUD = LibStub("AceAddon-3.0"):NewAddon("FusionHUD", "AceConsole-3.0");
local AceGUI = LibStub("AceGUI-3.0");

local _G = _G;

local FusionHUDBarPrototype = {};
FusionHUDBarPrototype.__index = FusionHUDBarPrototype;

local FusionHUDLockFrame = CreateFrame("Frame", nil, UIParent);


local FusionHUDBarType = { 
	"Health", 
	"Power",
	"Threat",
}

local FusionHUDLastUpdateClock = GetTime();
 

local FusionHUDBars = {};
local FusionHUDModSettings = {
	isUnlocked = false,
	transparency = 1,
	multiTransparency = false,
};

FusionHUDOptions = {
    name = "Fusion HUD",
    handler = FusionHUD,
    type = 'group',
    args = {
		newBar = {
			name = "New Bar",
			type = "execute",
			desc = "Creates a new HUD bar",
			order = 1,
			width = "full",
			func = 		function()
							local index = 0;
							table.foreach(FusionHUDBars, function(k,v) if index < v.barIndex then index = v.barIndex end end);
							FusionHUDBars["bar" .. index + 1] = FusionHUDBarPrototype:New(index + 1)
						end,
		},
		lockAll = {
			name = "Unlock All Bars",
			type = "execute",
			desc = "Unlocks all HUD bars for repositioning",
			order = 4,
			width = "full",
			func = 	function() 
						table.foreach(FusionHUDBars, 	function(k,v)
															v.draggable = true;
															v:LockHandler();
															FusionHUDModSettings.isUnlocked = true;
														end);
						FusionHUDLockFrame:Show();
						InterfaceOptionsFrame:Hide();
					end,
		},
		spacer1 = {
			name = "",
			type = "header",
			order = 3,
		},
		spacer2 = {
			name = "",
			type = "header",
			order = 5,
		},
		deleteBar = {
			name = "Delete Bar",
			order = 2,
			type = "execute",
			desc = "Deletes the last bar created",
			width = "full",
			confirm = function() return "Are you sure? This cannot be undone" end,
			func =	function()
						FusionHUD:DeleteBar() 
					end	
		},
		spacer3 = {
			name = "",
			type = "header",
			order = 6,
		},
		transparency = {
			name = "Transparency",
			type = "range",
			desc = "Set the HUD's overall transparency",
			min = 0,
			max = 100,
			step = 1,
			get = function() return FusionHUDModSettings.transparency*100 end,
			set = function(k,v) FusionHUDModSettings.transparency = v/100 end,
		},
--[[		indicators = {
			name = "Indicators",
			type = "group",
			order = 20,
			args = {},
		}, --]]
		bars = {
			name = "Bar List",
			order = 50,
			type = "group",
			desc = "Listing of each HUD bar",
			args = {}
		},
    },
};

function FusionHUD:OnInitialize()	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FusionHUD", FusionHUDOptions);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FusionHUD", "FusionHUD");
	
	SLASH_FUSIONHUD1, SLASH_FUSIONHUD2 = '/fusionhud', '/fh';
	
	FusionHUDMasterFrame = CreateFrame("Frame", nil, UIParent);
	
	FusionHUDMasterFrame:SetClampedToScreen(true);
	FusionHUDMasterFrame:SetPoint("CENTER",UIParent,"CENTER");
	FusionHUDMasterFrame:SetFrameStrata("LOW");
	FusionHUDMasterFrame:SetHeight(400);
	FusionHUDMasterFrame:SetWidth(400);
	FusionHUDMasterFrame:Show();
	FusionHUDMasterFrame:SetAlpha(1);

	FusionHUDLockFrame:SetClampedToScreen(true);
	FusionHUDLockFrame:ClearAllPoints();
	FusionHUDLockFrame:SetPoint("TOP", UIParent, "TOP", 0, -90);
	FusionHUDLockFrame:SetFrameStrata("FULLSCREEN");
	FusionHUDLockFrame:SetHeight(100);
	FusionHUDLockFrame:SetWidth(200);
	FusionHUDLockFrame.texture = FusionHUDLockFrame:CreateTexture(nil, "BACKGROUND");
	FusionHUDLockFrame.texture:SetAllPoints(FusionHUDLockFrame);
	FusionHUDLockFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", true);
	FusionHUDLockFrame.texture:SetVertexColor(0.7, 0.1, 0.1, 1);
	FusionHUDLockFrame.texture:SetTexCoord(0,1,0,1)
	FusionHUDLockFrame.text1 = FusionHUDLockFrame:CreateFontString(nil, "HIGH")
	FusionHUDLockFrame.text1:SetPoint("TOP", FusionHUDLockFrame, "TOP",0, -7);
	FusionHUDLockFrame.text1:SetFont("Fonts\\FRIZQT__.TTF", 24, "OUTLINE");
	FusionHUDLockFrame.text1:SetJustifyH("CENTER");
	FusionHUDLockFrame.text1:SetJustifyV("TOP");
	FusionHUDLockFrame.text1:SetHeight(30);
	FusionHUDLockFrame.text1:SetWidth(150);
	FusionHUDLockFrame.text1:SetText("FusionHUD");
	FusionHUDLockFrame.text2 = FusionHUDLockFrame:CreateFontString(nil, "HIGH")
	FusionHUDLockFrame.text2:SetPoint("TOP", FusionHUDLockFrame.text1, "BOTTOM",0, 0);
	FusionHUDLockFrame.text2:SetFont("Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf", 14);
	FusionHUDLockFrame.text2:SetJustifyH("CENTER");
	FusionHUDLockFrame.text2:SetJustifyV("TOP");
	FusionHUDLockFrame.text2:SetHeight(30);
	FusionHUDLockFrame.text2:SetWidth(150);
	FusionHUDLockFrame.text2:SetText("Use this button to relock your bars");
	
	FusionHUDLockFrame.lockButton = CreateFrame("Button", "lockButton", FusionHUDLockFrame, "UIPanelButtonTemplate")
	FusionHUDLockFrame.lockButton:SetPoint("TOP", FusionHUDLockFrame.text2, "BOTTOM", 0, 0);
	FusionHUDLockFrame.lockButton:SetWidth(100);
	FusionHUDLockFrame.lockButton:SetHeight(30);
	FusionHUDLockFrame.text1:SetVertexColor(1,1,.4,1);
	FusionHUDLockFrame.text2:SetVertexColor(1, 1, 1, 1);
	FusionHUDLockFrame.lockButton:SetText("Lock");
	FusionHUDLockFrame.lockButton:SetFrameStrata("FULLSCREEN_DIALOG");
	FusionHUDLockFrame.lockButton:SetScript('OnClick', 	function() 
															table.foreach(FusionHUDBars,	function(k,v) 
																								FusionHUDBars[k].draggable = false;
																								FusionHUDBars[k]:LockHandler();
																								FusionHUDModSettings.isUnlocked = false;
																							end) 
														end);
	FusionHUDLockFrame:Hide();
	
	FusionHUDMasterFrame:SetScript("OnUpdate", 	function() FusionHUD:OnUpdate()	end); 	
end

function SlashCmdList.FUSIONHUD(msg, editbox)
	InterfaceOptionsFrame_OpenToCategory(FusionHUD.optionsFrame);
end

function FusionHUD:OnEnable()
	local name = UnitName("player");
	if FusionHUDDB and FusionHUDDB[name] then
		FusionHUDBars = {};
		table.foreach(FusionHUDDB[name].settings, function(k,v)
											FusionHUDModSettings[k] = FusionHUDDB[name].settings[k]
										end);
		table.foreach(FusionHUDDB[name].bars, function(k,v)
											FusionHUDBars[k] = FusionHUDDB[name].bars[k];
											setmetatable(FusionHUDBars[k], FusionHUDBarPrototype);
											FusionHUDBars[k].lastValue = -99999999999;
											FusionHUDBars[k].lastValueMax = -99999999999;
											FusionHUDBars[k]:BuildBar();
										end);
		FusionHUDDB[name].bars = FusionHUDBars;
		FusionHUDDB[name].settings = FusionHUDModSettings;
	elseif FusionHUDDB then
		FusionHUDDB[name] = {};
		FusionHUDDB[name].settings = FusionHUDModSettings;
		FusionHUDDB[name].bars = FusionHUDBars;
	else
		FusionHUDDB = {};
		FusionHUDDB[name] = {};
		FusionHUDDB[name].settings = FusionHUDModSettings;
		FusionHUDDB[name].bars = FusionHUDBars;
		
	end
end	

function FusionHUD:MinimapButtonOnEnter()
	GameTooltip:SetOwner(FusionHUDMinimapButtonFrame, "ANCHOR_TOPRIGHT");
	GameTooltip:ClearLines();
	GameTooltip:SetAlpha(1);
	if FusionHUDLockFrame:IsVisible() then
		GameTooltip:SetText("FusionHUD |cffff9933" .. GetAddOnMetadata("FusionHUD", "Version") .. "|r", 1, .9, 0,  1, 1);
		GameTooltip:AddLine("|cffff3333Please relock your bars using the FusionHUD Lock Button");
	else
		GameTooltip:SetText("FusionHUD |cffff9933" .. GetAddOnMetadata("FusionHUD", "Version") .. "|r", 1, .9, 0,  1, 1);
		GameTooltip:AddLine("|cff44ff44Click to open FusionHUD Options", 1,1,1,1,1);
	end
	GameTooltip:Show();
end

function FusionHUD:MinimapButtonOnLeave(...)
	GameTooltip:Hide();
end

function FusionHUDBarPrototype:New(index)		--New Bar object, initializes the frames and textures
	local self = {};
    setmetatable(self, FusionHUDBarPrototype); -- Set the metatable so we used FusionHUDBarPrototype's __index
	
	self.barIndex = index;
	self.lastHeight = 400;
	self.lastWidth = 15;
	self.draggable = false;
	self.parentFrame = "FusionHUDMasterFrame";
	self.myPoint = "CENTER";
	self.parentPoint = "CENTER";
	self.offsetX = 0;
	self.offsetY = 0;
	self.lastOrientation = "Left"
	self.orientation = "Left";
	self.textureName = "Interface\\AddOns\\FusionHUD\\Graphics\\Straight Bar Glass";
	self.font = "Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf";
	self.fontKey = "FrizQuadrata";
	self.fontSize = 14;
	self.height = 400;
	self.width = 15;
	self.barType = "Health";
	self.barTarget = "player";
	self.totals = true;
	self.text = "EEEEEEE";
	self.warningColor = false;
	self.lastValue = -999999999999999;
	self.lastValueMax = -9999999999999999;
	self.lastTarget = nil;

	self:BuildBar();
	
	return self;
end

function FusionHUDBarPrototype:LockHandler()
	if self.draggable == true then
		self.frame.titleRegion:SetAllPoints(self.frame);
		self.frame:SetBackdrop({bgFile = "Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", tile = true, tilesize = self.width});
		self.frame:SetBackdropColor(0.8, 0.8, 0.8, 1);
		self.frame:Show();
		self.frame:SetAlpha(1);
		self.frame:EnableMouse(true);
		self.frame:SetMovable(true);
		return
	else
		self.myPoint, self.parentFrame, self.parentPoint, self.offsetX, self.offsetY = self.frame:GetPoint();
		self.frame.titleRegion:ClearAllPoints();
		self.frame:SetBackdrop({nil});
		self.frame:EnableMouse(false);
		self.frame:SetMovable(false);
--		self.frame.dragTexture:ClearAllPoints()
		if FusionHUDLockFrame:IsShown() then
			FusionHUDLockFrame:Hide();
			InterfaceOptionsFrame:Show();
		end
	end
end

function FusionHUDBarPrototype:Update()	-- The bar update function
	local barL, barR;
	if FusionHUDModSettings.isUnlocked then
		FusionHUDMasterFrame:SetAlpha(1);
		return
	end
	FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.transparency);
	if UnitName(self.barTarget) == nil then
		self.frame:Hide();
		return
	else
		self.frame:Show();
		if self.orientation == "Left" then
			barL = 0;
			barR = 1;
		else
			barL = 1;
			barR = 0;
		end
		if self.barType == FusionHUDBarType[1] then		--check for type of bar, power or health
			-- check to see if dimension or health/power values have changed
			if self.lastValue ~= UnitHealth(self.barTarget) or 
			   self.lastValueMax ~= UnitHealthMax(self.barTarget) or 
			   self.height ~= self.lastHeight or 
			   self.width ~= self.lastWidth or
			   self.orientation ~= self.lastOrientation then
				local health = UnitHealth(self.barTarget) .. "/" .. UnitHealthMax(self.barTarget);
				self.lastHeight, self.lastWidth, self.lastValue, self.lastValueMax, self.lastOrientation = self.height, self.width, UnitHealth(self.barTarget), UnitHealthMax(self.barTarget), self.orientation;
				self.frame.containerTexture:SetTexCoord(barL, barR, 1, 0);
				--set color for the health bar
				self.frame.barTexture:SetVertexColor(0,0.8,0,1);
				self.frame.text:SetVertexColor(0,0.8,0,1);
				--set bar height and text for a health bar
				if (self.height*UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget)) > 0 then
					self.frame.barTexture:SetHeight(self.height*UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget));
					self.frame.barTexture:SetTexCoord(barL,barR,UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget),0);
				elseif (self.height*UnitPower(self.barTarget)/UnitHealthMax(self.barTarget)) ==0 then
					self.frame.barTexture:SetHeight(1);
					self.frame.barTexture:SetTexCoord(barL,barR,UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget),0);
				else
					self.frame.barTexture:SetHeight(1);
					self.frame.barTexture:SetTexCoord(barL,barR,0,0);
				end				
				self.frame.text:SetText(health);
			end
		elseif self.barType == FusionHUDBarType[2] then
			local powerType = UnitPowerType(self.barTarget);
			-- check to see if values have changed
			if self.lastValue ~= UnitPower(self.barTarget) or 
			   self.lastValueMax ~= UnitPowerMax(self.barTarget) or 
			   self.height ~= self.lastHeight or 
			   self.width ~= self.lastWidth or
			   self.orientation ~= self.lastOrientation then
				local power = UnitPower(self.barTarget) .. "/" .. UnitPowerMax(self.barTarget);
				self.lastValue, self.lastValueMax, self.lastHeight, self.lastWidth, self.lastOrientation = UnitPower(self.barTarget), UnitPowerMax(self.barTarget), self.height, self.width, self.orientation;
				self.frame.containerTexture:SetTexCoord(barL, barR, 1, 0);
				--set color for the power bar
				if powerType == 1 then
					self.frame.barTexture:SetVertexColor(0.8,0,0,1);
					self.frame.text:SetVertexColor(0.8,0,0,1);
				elseif powerType == 6 then
					self.frame.barTexture:SetVertexColor(0.6,0,0,1);
					self.frame.text:SetVertexColor(0.6,0,0,1);
				elseif powerType == 0 then
					self.frame.barTexture:SetVertexColor(0.2,0.4,1,1);
					self.frame.text:SetVertexColor(0.2,0.3,1,1);
				elseif powerType == 2 then
					self.frame.barTexture:SetVertexColor(0.8,0.3,0,1);
					self.frame.text:SetVertexColor(0.8,0.3,0,1);
				else
					self.frame.barTexture:SetVertexColor(1,1,0,1);
					self.frame.text:SetVertexColor(1,1,0,1);
				end
				--set bar height and text for the power bar
				if (self.height*UnitPower(self.barTarget)/UnitPowerMax(self.barTarget)) > 0 then
					self.frame.barTexture:SetHeight(self.height*UnitPower(self.barTarget)/UnitPowerMax(self.barTarget));
					self.frame.barTexture:SetTexCoord(barL,barR,UnitPower(self.barTarget)/UnitPowerMax(self.barTarget),0);
				elseif (self.height*UnitPower(self.barTarget)/UnitPowerMax(self.barTarget)) ==0 then
					self.frame.barTexture:SetHeight(1);
					self.frame.barTexture:SetTexCoord(barL,barR,UnitPower(self.barTarget)/UnitPowerMax(self.barTarget),0);
				else
					self.frame.barTexture:SetHeight(1);
					self.frame.barTexture:SetTexCoord(barL,barR,0,0);
				end
				self.frame.text:SetText(power);
			end
		elseif self.barType == FusionHUDBarType[3] then
			self.frame.containerTexture:SetTexCoord(barL, barR, 1, 0);
			if UnitName("target") ~= nil and UnitName("target") ~= "Unknown" then
				local isTanking, status, threatPct, rawThreatPct, threatValue = UnitDetailedThreatSituation(self.barTarget, "target");
				self.frame.containerTexture:SetTexCoord(barL, barR, 1, 0);
				if self.lastValue ~= threatPct or self.lastTarget ~= UnitGUID("target") or UnitAffectingCombat(self.barTarget) then
					self.lastValue = threatPct;
					local r, g, b = GetThreatStatusColor(status);
					self.lastTarget = UnitGUID("target");
					local B = tonumber(self.lastTarget:sub(5,5), 16);
					local maskedB = B % 8;  -- determines if the target is a player, pet or vehicle. Threat bar will filter these out.
					local knownTypes = {[0]="player", [3]="NPC", [4]="pet", [5]="vehicle"};
					if knownTypes[maskedB] ~= "player" or knownTypes[maskedB] ~= "pet" or knownTypes[maskedB] ~= "vehicle" then
						if threatPct == nil or threatPct <= 0 then
							self.frame:Hide();
							self.frame.barTexture:SetHeight(1);
							self.frame.barTexture:SetTexCoord(barL,barR,0,0);
						elseif threatPct > 100 then
							self.frame:Show();
							self.frame.barTexture:SetHeight(self.height);
							self.frame.barTexture:SetTexCoord(barL,barR, 1, 0);
						elseif threatPct > 0 then
							self.frame:Show();
							self.frame.barTexture:SetHeight(self.height*threatPct/100);
							self.frame.barTexture:SetTexCoord(barL,barR,threatPct/100,0);
						end
					end
					self.frame.barTexture:SetVertexColor(r, g, b, 1);  -- color key:  Gray: safely low threat, Yellow: Close to the tank on threat, Orange: (tanking) Losing aggro soon, Red:  Has aggro, tanking
					self.frame.text:SetText(math.floor(threatPct or 0) .. "%");
				else
					self.frame:Hide();  
				end 
			else
				self.frame:Hide()
			end
		end
	end	
end

function FusionHUDBarPrototype:BuildBar()
	local barL, barR;
	if self.font == nil then
		self.font = "Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf";
		self.fontKey = "Droid Sans";
		self.fontSize = 14;
	end
	if self.textureName == "Interface\\AddOns\\FusionHUD\\Graphics\\White Straight Bar Base" then
		self.textureName = "Interface\\AddOns\\FusionHUD\\Graphics\\Straight Bar Glass";
	end
	if self.orientation == "Left" then
		barL = 0;
		barR = 1;
	else
		barL = 1;
		barR = 0;
	end
	if self.lastTarget == nil then
		self.lastTarget = 0;
	end
	self.frame = nil;
	--Set up bar frame
	self.frame = CreateFrame("Frame", nil, FusionHUDMasterFrame);
	self.frame:ClearAllPoints();
	self.frame:SetPoint(self.myPoint, FusionHUDMasterFrame, self.parentPoint);
	self.frame:SetHeight(self.height);
	self.frame:SetWidth(self.width);
	self.frame:SetClampedToScreen(true);
	--Set Up the title region for the bar to enable dragging
	self.frame.titleRegion = self.frame:CreateTitleRegion();
	self.frame.titleRegion:ClearAllPoints();
	--Set up the bar texture
	self.frame.barTexture = self.frame:CreateTexture(nil, "BACKGROUND");
	self.frame.barTexture:SetTexture(self.textureName, false);
	self.frame.barTexture:ClearAllPoints();
	self.frame.barTexture:SetPoint("BOTTOM", self.frame, "BOTTOM");
	self.frame.barTexture:SetHeight(self.height);
	self.frame.barTexture:SetWidth(self.width);
	--Set up the bar background/scale texture
	self.frame.containerTexture = self.frame:CreateTexture(nil, "OVERLAY");
	self.frame.containerTexture:SetTexture(self.textureName .. " Container", false);
	self.frame.containerTexture:ClearAllPoints();
	self.frame.containerTexture:SetPoint("BOTTOM", self.frame, "BOTTOM");
	self.frame.containerTexture:SetHeight(self.height);
	self.frame.containerTexture:SetWidth(self.width);
	self.frame.containerTexture:SetTexCoord(barL,barR,1,0);
	self.frame.containerTexture:SetBlendMode("BLEND");
	--Set up bar text
	self.frame.text = self.frame:CreateFontString(nil, "LOW");
	self.frame.text:ClearAllPoints();
	self.frame.text:SetPoint("TOP", self.frame, "BOTTOM");
	self.frame.text:SetFont(self.font, self.fontSize, "OUTLINE");
	self.frame.text:SetJustifyH("CENTER");
	self.frame.text:SetJustifyV("TOP");
	self.frame.text:SetHeight(30);
	self.frame.text:SetWidth(150);
	
	self.frame:SetPoint(self.myPoint, self.parentFrame, self.parentPoint, self.offsetX, self.offsetY);
	
	--Set Up the bar's options table
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex] = {};
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].name = "Bar " .. self.barIndex;
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].type = "group";
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].order = self.barIndex;
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].args = {
		header = {
			name = "Bar " .. self.barIndex,
			order = 1,
			type = "header"
		},		
		lock = {
			name = "Unlock",
			order = 2,
			type = "execute",
			desc = "Unlocks the bar for repositioning",
			func = 	function() 
						self.draggable = true;
						self:LockHandler()
						FusionHUDLockFrame:Show();
						InterfaceOptionsFrame:Hide();
						FusionHUDModSettings.isUnlocked = true;
					end;
		},
		barType = {
			name = "Bar Type",
			order = 11,
			type = "select",
			desc = "The bar can track either Health or Power(Mana/Runic Power/Energy/Focus)",
			style = "dropdown",
			values = {
				Power = "Power", 
				Health = "Health",
				Threat = "Threat",
			},
			get = 	function() return FusionHUDBars["bar" .. self.barIndex].barType end,
			set = 	function(key, value) FusionHUDBars["bar" .. self.barIndex].barType = value end
		},
		barTarget = {
			name = "Bar Target",
			order = 10,
			type = "select",
			desc = "The unit that the bar will track",
			style = "dropdown",
			values = {
				player = "Player",
				pet = "Pet",
				playertarget = "Player target",
				playertargettarget = "Target of Target",
				pettarget = "Pet Target"
			},
			get = 	function() return self.barTarget end,
			set = 	function(key, value) self.barTarget = value end
		},
		barSkin = {
			name = "Bar Skin",
			order = 11,
			type = "select",
			desc = "Select the bar's skin",
			style = "dropdown",
			values = {
				["Interface\\AddOns\\FusionHUD\\Graphics\\Curved Bar"] = "Curved",
				["Interface\\AddOns\\FusionHUD\\Graphics\\Straight Bar Glass"] = "Straight Glass",
			},
			get = 	function() return self.textureName end,
			set = 	function(k,v)
						self.textureName = tostring(v);
						self.frame.barTexture:SetTexture(self.textureName, false);
						self.frame.containerTexture:SetTexture(self.textureName .. " Container", false)
					end,
		},	
		barFacing = {
			name = "Bar Facing",
			order = 12,
			type = "select",
			desc  = "Select which way the bar faces",
			style = "radio",
			values = {
				["Left"] = "Left",
				["Right"] = "Right"
			},
			get = 	function() return self.orientation end,
			set = 	function(k,v)
						self.orientation = tostring(v);
					end
		},				
		barHeight = {
			name = "Bar Height",
			order = 13,
			type = "range",
			min = 50,
			max = 1200,
			step = 1,
			get = 	function() return self.height end,
			set = 	function(k,v) 
						self.height = v; 
						self.frame:SetHeight(self.height); 
						self.frame.containerTexture:SetHeight(self.height);
						self.frame.barTexture:SetHeight(self.height);
					end
		},
		barWidth = {
			name = "Bar Width",
			order = 14,
			type = "range",
			min = 1,
			max = 125,
			step = 1,
			get = 	function() return self.width end,
			set = 	function(k,v) 
						self.width = v; 
						self.frame:SetWidth(self.width); 
						self.frame.containerTexture:SetWidth(self.width);
						self.frame.barTexture:SetWidth(self.width);
					end
		},
		fontHeader = {
			name = "Font Settings",
			type = "header",
			order = 20
		},
		font = {
			name = "Font",
			type = "select",
			order = 21,
			desc = "Choose the font this bar will use",
			style = "dropdown",
			values = {
				["Fonts\\FRIZQT__.TTF"] = "FrizQuadrata",
				["Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf"] = "Droid Sans",
				["Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans-Bold.ttf"] = "Droid Sans Bold",
				["Interface\\AddOns\\FusionHUD\\Fonts\\nobile.ttf"] = "Nobile" 
			},
			get = 	function() return tostring(self.font) end,
			set = 	function(k,v)
						self.font = v;
						self.fontKey = k;
						self.frame.text:SetFont(self.font, self.fontSize, "OUTLINE");
					end
		},
		fontSize = {
			name = "Font Size",
			type = "range",
			order = 25,
			desc = "Select font size",
			min = 6,
			max = 24,
			step = 1,
			get = 	function() return self.fontSize end;
			set = 	function(k,v)
						self.fontSize = v;
						self.frame.text:SetFont(self.font, self.fontSize, "OUTLINE");
					end
		}
	}
end

function FusionHUD:DeleteBar()
	local index = -1;
	table.foreach(FusionHUDBars, function(k,v) if index < v.barIndex then index = v.barIndex end end);
	if index then
		FusionHUDOptions.args.bars.args["bar" .. index] = nil;
		FusionHUDBars["bar" .. index].frame:Hide();
		FusionHUDBars["bar" .. index] = nil;
	else
		FusionHUDBars = {};
	end
end

function FusionHUD:OnUpdate()
	local FusionHUDNextUpdateClock = GetTime()
	if FusionHUDNextUpdateClock - FusionHUDLastUpdateClock > 0.1 then
		FusionHUDLastUpdateClock = FusionHUDNextUpdateClock;
		table.foreach(FusionHUDBars, 	function(k,v) 
											FusionHUDBars[k]:Update() 
--											FusionHUD:Print(k .. " " .. v);
										end);
	end
end